use std::collections::HashSet;

use sqlx::query_as;

use terminwahl_typen::{
    AppointmentSlot, AppointmentSlots, IdType, SlotId, Subject, Subjects, Teacher, Teachers,
};

use super::Pool;

pub async fn get_teachers(db: &Pool) -> Result<Teachers, sqlx::Error> {
    query_as!(
        Teacher,
        r#"
        SELECT *
        FROM `teachers`"#,
    )
    .fetch_all(db)
    .await
}

pub async fn get_teacher_by_id(db: &Pool, teacher_id: IdType) -> Result<Teacher, sqlx::Error> {
    query_as!(
        Teacher,
        r#"
        SELECT *
        FROM `teachers` WHERE id = ?"#,
        teacher_id
    )
    .fetch_one(db)
    .await
}

pub async fn get_subjects(db: &Pool) -> Result<Subjects, sqlx::Error> {
    query_as!(
        Subject,
        r#"
        SELECT *
        FROM `subjects`"#,
    )
    .fetch_all(db)
    .await
}

pub async fn get_slots(db: &Pool) -> Result<AppointmentSlots, sqlx::Error> {
    match query_as!(
        AppointmentSlot,
        r#"
        SELECT *
        FROM `appointment_slots`"#,
    )
    .fetch_all(db)
    .await
    {
        Ok(elems) => Ok(elems.into_iter().map(|e| (e.id, e)).collect()),
        Err(e) => Err(e),
    }
}
pub async fn get_slot_by_id(db: &Pool, slot_id: IdType) -> Result<AppointmentSlot, sqlx::Error> {
    match query_as!(
        AppointmentSlot,
        r#"
        SELECT *
        FROM `appointment_slots` WHERE id = ?"#,
        slot_id
    )
    .fetch_one(db)
    .await
    {
        Ok(slot) => Ok(slot),
        Err(e) => Err(e),
    }
}
pub async fn get_unavailable(db: &Pool) -> Result<HashSet<SlotId>, sqlx::Error> {
    match query_as!(
        SlotId,
        r#"
        SELECT teacher_id, slot_id
        FROM `appointments` WHERE datetime(expires) > datetime('now') OR expires is NULL;"#,
    )
    .fetch_all(db)
    .await
    {
        Ok(elems) => Ok(elems.into_iter().collect()),
        Err(e) => Err(e),
    }
}
